#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed Jan. 2008 by Fredo6

# This software is provided as an example of using the Ruby interface to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name		:   LibPlaneSelectorTool.rb
# Type		:   Utility Library as a Tool
# Description	:   Allow to select graphically a Plane
# Menu Item	:   none
# Context Menu	:   none
# Usage		:   See Tutorial 
# Date		:   24 Jan 08
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************

require 'sketchup.rb'
require 'libtraductor.rb'		# for language translation

module LibPlaneSelectorTool

STATE_ORIGIN = 0
STATE_EXECUTION = 1

MSG_Degree = "Degree |FR| Degr\"
MSG_PlaneAngle = "Plane Angle |FR| Angle du plan"
MSG_Input_Origin = ["Select Origin and Plane (Shift to lock plane)",
                    "|FR| S\lectionner l'origine et le plan (Maj pour verrouiller le plan)"]
MSG_Input_Execution = ["Double Click or Enter to launch operation '%1' (or adjust angle value in VCB)",
                    "|FR| Double cliquez ou Enter pour ex\cuter l'op\ration '%1' (ou valeur de l'angle dans la VCB)"]
					
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Tool to select a plane, direction and angle - More or less mimic Skecthup Protractor
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class PlaneSelectorTool

def initialize(caller_class, cursorpath=nil, opname="")
	Traductor.load_translation LibPlaneSelectorTool, /MSG_/, binding, "@msg_"
	@caller = caller_class
	@cursorpath = cursorpath
	@opname = opname
	@idcursor = 0
	@ip_origin = Sketchup::InputPoint.new
	@origin = ORIGIN
	@axis_def = Y_AXIS.reverse
	@angle_def = 0.0
	@normal_def = @axis_def
	@normal = @axis_def
	@pt_ref = ORIGIN
	@gray_color = Sketchup::Color.new "Gray"
	@face_color = @gray_color
	
	#defining the Plane grid
	nv = 4
	leng = 200.cm
	step = leng / nv
	@lptgrid = []
	for i in -nv..nv
		@lptgrid.push Geom::Point3d.new(-leng, i * step, 0), Geom::Point3d.new(leng, i * step, 0)
		@lptgrid.push Geom::Point3d.new(i * step, -leng, 0), Geom::Point3d.new(i * step, leng, 0)
	end
			
	self.compute_origin
	
	if (@cursorpath)
		f = Sketchup.find_support_file @cursorpath, "Plugins"
		@idcursor = UI::create_cursor f, 10, 10
	end	
end

def activate
	set_state STATE_ORIGIN
end

def deactivate(view)
	view.invalidate
end

def onSetCursor
	UI::set_cursor @idcursor if (@state == STATE_EXECUTION && @idcursor != 0)
end

def call_execute
	puts "method call"
	return unless @caller
	hmeth = @caller.method 'plane_selector_execute'
	if (hmeth == nil || hmeth.arity != 2)
		UI.beep
	else
		begin
			if hmeth.call @origin, @normal
				Sketchup.active_model.select_tool nil 
				return
			end	
		rescue			
		end	
	end
end

def onLButtonDoubleClick(flags, x, y, view)
	if @state == STATE_EXECUTION
		call_execute
	else
		UI.beep
	end
end

def onKeyDown(key, rpt, flags, view)
	key = Traductor.check_key key, flags, false
	case key
	when VK_UP
		newnormal = Z_AXIS
	when VK_RIGHT
		newnormal = X_AXIS	
	when VK_LEFT
		newnormal = Y_AXIS.reverse	
	when VK_DOWN
		newnormal = @axis_def
	else
		return false
	end
	@axis_def = newnormal
	if (@normal == @normal_def)
		@normal = @axis_def
		self.compute_origin
		view.invalidate
	end	
	@normal_def = @axis_def
	@angle_def = 0.0
	info_angle @angle_def
end

def onKeyUp(key, rpt, flags, view)
	key = Traductor.check_key key, flags, true
	case key
	when 13			#Return key
		unless @usertext
			return UI.beep if @state < STATE_EXECUTION
			call_execute
		end	
		@usertext = false
	else
		return false
	end	
	true
end

def onUserText(text, view)
	@usertext = true
	angle = parse_angle text
	return UI.beep unless angle
	
	case @state
	when STATE_ORIGIN
		@angle_def = angle
		@normal_def = Geom::Transformation.rotation(@origin, @axesD[0], @angle_def) * @axis_def
		@normal = @normal_def
		self.compute_origin
	end
	view.invalidate
	info_angle angle
	set_state @state + 1 unless @state == STATE_ORIGIN
end

def parse_angle (text)
	text = text.strip
	if text =~ /^=/
		st = $'
		begin
			if st =~ /d$/i
				sd = formula_float $`
				dangle = Kernel::eval sd
			elsif st =~ /r$/i
				sd = formula_float $`
				angle = (Kernel::eval sd)
				dangle = angle.to_f * 180 / Math::PI
			else
				st = formula_float(st)
				tg = Kernel::eval st
				dangle = (tg == 0.0) ? 0.0 : ((Math.atan2(tg, 1)) * 180 / Math::PI)
			end	
		rescue
			UI.beep
			return false
		end
	elsif text =~ /^(d|.)/
		dangle = text.to_f
	else
		UI.beep
		return false
	end	
	dangle = dangle.modulo 360.0
	angle = dangle * Math::PI / 180
end

def formula_float(s)
	s2 = s.gsub(/\.\d/) { |m| ($`[-1..-1] =~ /\d/) ? m : '0' + m }
	s2.gsub(/\d+/) { |m| m + ((($`[-1..-1] != '.') && ($'[0..0] != '.')) ? '.0' : '') }
end

#Control the 4 states of the tool
def set_state(state)
	@state = state
	@state = STATE_EXECUTION if (@state > STATE_EXECUTION)
	info_show
end

def onLButtonDown(flags, x, y, view)
	set_state @state + 1
end

#Handle Escape key
def onCancel(flag, view)
	return unless flag == 0
	return Sketchup.active_model.select_tool(nil) if (@state == STATE_ORIGIN)
	set_state @state - 1
end

def compute_origin
	@axesD = @normal.axes
	@tt = Geom::Transformation.axes(@origin, @axesD[0], @axesD[1], @axesD[2]) 	
	@color = get_color @normal
end

#OnMouseMove method for Tool
def onMouseMove(flags, x, y, view)
	case @state	
	when STATE_ORIGIN		#input Origin and Plane
		#view.lock_inference
		@ip_origin.pick view, x, y
		view.tooltip = @ip_origin.tooltip
		face = @ip_origin.face
		@origin = @ip_origin.position
		unless (flags == 4)		#unless Shift Key is down, to lock plane
			if (face)
				ph = view.pick_helper
				ph.do_pick x,y
				best = ph.best_picked
				if (best && best.typename == 'ComponentInstance')
					@normal = best.transformation * (face.normal)
				else
					@normal = @ip_origin.transformation * (face.normal)
				end	
				@face_color = (face.material) ? face.material.color : @gray_color
			else
				@normal = @normal_def
			end
		end	
		self.compute_origin
		view.invalidate
	end	
end

#Draw method for tool
def draw(view)
	if (@state >= STATE_ORIGIN)
		#Compute the right scale to keep the protractor the same size
		size = view.pixels_to_model 1, @origin 	
		t = @tt * Geom::Transformation.scaling(size)
		
		#draw the origin
		@ip_origin.draw view

		#draw the plane grid
		view.drawing_color = @color 
		pts = []
		@lptgrid.each {|pt| pts.push (t * pt)}
		view.draw_lines pts
	end
end

#display angle value in the VCB
def info_angle(angle)
	dangle = angle * 180 / Math::PI
	dangle = dangle.modulo 360.0
	Sketchup.set_status_text sprintf("%3.1f ", dangle) + @msg_Degree, SB_VCB_VALUE
end

#display information in the Sketchup status bar
def info_show
	case @state
	when STATE_ORIGIN
		msg = @msg_Input_Origin
		label = @msg_PlaneAngle
	when STATE_EXECUTION
		msg = Traductor[MSG_Input_Execution, @opname]
		label = @msg_PlaneAngle
	end
	Sketchup.set_status_text msg
	Sketchup.set_status_text label, SB_VCB_LABEL
end
	
#Get color corresponding to a vector direction
def get_color vec
	if (vec == nil || vec.length == 0)
		colorname = "Black"
	elsif (vec.parallel? X_AXIS)
		colorname = "Red"
	elsif (vec.parallel? Y_AXIS)
		colorname = "Lawngreen"
	elsif (vec.parallel? Z_AXIS)
		colorname = "Blue"
	else
		colorname = "Black"
	end		
	color = adjust_color colorname
end

def adjust_color(colorname)
	prox = 25
	color = Sketchup::Color.new colorname
	return color if ((color.red - @face_color.red).abs > prox)
	return color if ((color.blue - @face_color.blue).abs > prox)
	return color if ((color.green - @face_color.green).abs > prox)
	color = Sketchup::Color.new 255 - color.red, 255 - color.green, 255 - color.blue 
end

end #Class PlaneSelectorTool

end #Module LibPLaneSelectorTool